;
;	DISEQC_SW_629noled.ASM
;	DISEQC decoder for a latching relay
;	PIC12F629 w/comparitor
;	4 MHz internal osc
;	Switches Diseqc ports 1 and 2 on command
;	PNP switches w/inverted drive
;	More agressive error detect
;  	8/30/12			
;	Repeat msg filter
;	No LED support - faster error recovery			
;------------------------------------------------------------
	LIST	P=12F629	
	ERRORLEVEL	-302
	include	<P12F629.INC>
	__CONFIG	h'31A4'
;------------------------------------------------------------	
;		Port Setup
;					
;	GPIO	0	1  	2	3       4       5
;		+In 	-In  	Out 	/MCLR	RESET	SET
;				Out	In	Out	Out
;------------------------------------------------------------
;
	ORG	0
	goto	main
;
;------------------------------------------------------------
        cblock  0x20
	buffer_0		;(20h) buffer 
        buffer_1                ;
        buffer_2                ;
	buffer_3		;
	buffer_4		;
;	
        util_count		;
        bit_count              	;bit counter
        zcd_count             	;zero crossings cnt
	delay_count		;delays
	lull_count		;no activity TO count
	cca_count		;
	ccb_count		;(2Bh)
	endc
;------------------------------------------------------------
;
;
#define	RRSET		GPIO,4		;relay reset
#define	RSET		GPIO,5		;relay set
;
;	Bytewise match E0 10 38 F0 w/parity
;
C_0	equ	b'00001110'		;7-4 unused
C_1	equ	b'00000000'		;5/3
C_2	equ	b'10000000'		;6/2
C_3	equ	b'11100001'		;7/1
C_4	equ	b'11100000'		;8/0
;
;
MS_10		equ	.40		;10 mS
MS_60		equ	.240		;60 mS	
;
;------------------------------------------------------------
;
main
	clrf	GPIO
	movlw	b'00000010'		;	
	movwf	CMCON			;comparitor - w/ no output
	clrwdt				;
	bsf	STATUS,RP0		;bank 1
	movlw	b'00111011'		;
	movwf	TRISIO			;GPIO:2 output
	clrwdt
	movlw	b'10001000'
	movwf	OPTION_REG		;TMR0/1, WPU disable
	movlw	b'00000000'
	movwf	WPU			;no weak pullups
	movlw	b'00000000'
	movwf	IOC			;no int on change	
        movlw   b'00000000'             ;
        movwf   PIE1                    ;no perif ints
	call	3FFh
	movwf	OSCCAL			;calibrate int osc
	bcf	STATUS,RP0		;bank 0
;
	movlw	b'00000001'		;
	movwf	T1CON			;T1/1, on
	clrf	PIR1			;
;
        movlw   b'00000000'             ;ints off
	movwf	INTCON			;
	bsf	STATUS,RP0		;bank 1
	movlw	b'00001011'		;
	movwf	TRISIO			;GPIO:2, 4, 5 outputs
	bcf	STATUS,RP0		;bank 0
	bsf	RSET			;relay controls high
	bsf	RRSET			;PNP relay driver
	clrf	cca_count		;
	clrf	ccb_count		;
;
;------------------------------------------------------------
;
;------------------------------------------------------------
;	4 MHz Clock
;	22 kHz tone has a zero crossing interval = 22.72 uS
;	Worst case is 14 uS per loop
;
start_here
	movlw	0x20			;(1)
	movwf	FSR			;(1)reset FSR pointer
house_clean_loop
	clrf	INDF			;(1)clear named registers
	incf	FSR,W			;(1)
	sublw	0x29 			;(1)
	bnc	tone_detect		;(2,3)
	incf	FSR,F			;(1)
	goto	house_clean_loop	;(2)
;
;	Houseclean above - Data detect below
;
tone_detect
	clrf	zcd_count		;(1)reset
	clrf	lull_count		;(1)ditto
	movf	CMCON,F			;(1)read & reset
	bcf	PIR1,CMIF		;(1)clear compare flag
;	
holdoff_TO
	movlw	.156			;(1)100 uS to rollover Time Out
	movwf	TMR0			;(1)
	bcf 	INTCON,T0IF		;(1)insure reset
;
check_ints
	btfsc	INTCON,T0IF		;(1/2)timeout?
	goto	zcd_lull		;(2)yes, tone done
	btfss	PIR1,CMIF		;(1/2)comparitor interrupt?
	goto	check_ints		;(2)loop
;
comp_handler
	movf	CMCON,F			;(1)read & reset
	bcf	PIR1,CMIF		;(1)
	incf	zcd_count,F		;(1)update count
	goto	holdoff_TO		;(2)loop back
;
zcd_lull
	movf	zcd_count,F		;test for too short
	bnz	stash_loot		;
	incf	lull_count,F		;no ints counted +1
	btfss	lull_count,5		;>32? (3.2mS)
	goto	holdoff_TO		;no
	goto	interprit		;exit capture <<>>
;
stash_loot
	movfw	zcd_count		;
	sublw	.16			;
	skpnc				;
	goto	tone_detect		;<17 xings
;
check_cycles
	movlw	.27			;test <= 26 xings
	subwf	zcd_count,W		;
	skpnc				;
	goto	test_4_0		;too long for a 1, test for 0
 	setc				;call it a one
	goto	roll_in_bit		;exit & store
;
test_4_0
	movlw	.36			;test => 36 xings
	subwf	zcd_count,W		;
	skpc
	goto	tone_detect		;too few cycles for a 0
	movlw	.52			;<52 cycles?
	subwf	zcd_count,W		;
	skpnc
	goto	tone_detect		;too long
	clrc				;call it a zero
roll_in_bit
	rlf	buffer_0,F		;roll carry into buffer_0-4
	rlf	buffer_1,F		;we're looking for
	rlf	buffer_2,F		;4 bytes 
	rlf	buffer_3,F		;and parity
	rlf	buffer_4,F		;last to fill		
	incf	bit_count,F		;count bits
	movfw	bit_count		;count to W
	xorlw	.4			;
	skpz
	goto	tone_detect		;back to get next tone
;
	movfw	buffer_0		;1st nybble
	andlw	b'00001111'		;
	xorlw	b'00001110'		;is it 0Eh?
	skpnz
	goto	tone_detect		;yes, back to get next tone
	goto	start_here		;no, bail and start over
;
;------------------------------------------------------------
;
;	Looks for bit/bytewise match
;
interprit
	movlw	.36			;4 bytes + 4 parity bits
	xorwf	bit_count,W		;
	skpz
	goto	start_here		;
;
	movfw	buffer_4		;compare buffer_0
	xorlw	C_0			;to C_0
	skpz
	goto	start_here		;no match
;	
	movfw	buffer_3		;compare buffer_1
	xorlw	C_1			;to C_1
	skpz
	goto	start_here		;no match
;
	movfw	buffer_2		;compare buffer_2
	xorlw	C_2			;to C_2
	skpz
	goto	start_here		;no match
;
	movfw	buffer_1		;compare buffer_3
	xorlw	C_3			;to C_3
	skpz
	goto	start_here		;wrong
;
	movlw	b'00011000'		;
	andwf	buffer_0,F		;mask all but bits 3,4
	skpnz
	goto  	SW_pos_A		;= 0
	movlw	.8			;
	xorwf	buffer_0,W		;
	skpz				;position B
	goto	mini_break		;do nothing	
;
SW_pos_B
	clrf	cca_count		;insure opp is reset
	movf	ccb_count,F		;test ccb_count for 0
	bz	action_b		;
	incf	ccb_count,F		;
	goto	mini_break		;
;
action_b
	bcf	RRSET			;reset on
	call	delay_10mS		;on for 10 mS
	bsf	RRSET			;off
	incf	ccb_count,F		;increment counter
	goto	mini_break		;
;
;
SW_pos_A
	clrf	ccb_count		;insure opp is reset
	movf	cca_count,F		;test cca_count for 0
	bz	action_a		;=0, hit reset
	incf	cca_count,F		;
	goto	mini_break		;
;
action_a
	bcf	RSET			;set on
	call	delay_10mS		;on for 10 mS
	bsf	RSET			;pulse done 
	incf	cca_count,F		;increment counter
;
mini_break
;	call	delay_10mS		;wait 10 mS
	goto	start_here		;loop to start
;
;------------------------------------------------------------
;
;
;       Subroutines area
;
;------------------------------------------------------------
;
;
;------------------------------------------------------------
;	Delays
;
delay_10mS
	movlw	MS_10
	movwf	delay_count
	goto	T1_delay_loop
;
delay_60mS
	movlw	MS_60	
	movwf	delay_count
;
T1_delay_loop
	clrf	TMR1L			;
	movlw	0xff			;
	movwf	TMR1H			;
	bcf	PIR1,TMR1IF		;256 uS per rollover
T1_loopA
	btfss	PIR1,TMR1IF
	goto	T1_loopA		;loop til roll over
	decfsz	delay_count,F		;count down
	goto	T1_delay_loop		;loop
	return
;	
;------------------------------------------------------------
        END
;------------------------------------------------------------
;
;Copyright (c) 2012, Mike Berg aka N0QBH
;All rights reserved.
;
;Redistribution and use in source and binary forms, with or without modification, 
;are permitted provided that the following conditions are met:
;
;Redistributions of source code must retain the above copyright notice, 
;this list of conditions and the following disclaimer. 
;Redistributions in binary form must reproduce the above copyright notice, 
;this list of conditions and the following disclaimer in the documentation 
;and/or other materials provided with the distribution. 
;Neither the name of the <ringolake.com> nor the names of its contributors 
;may be used to endorse or promote products derived from this software without 
;specific prior written permission. 
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
;AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
;THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
;PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
;BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
;CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
;SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
;INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
;CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
;ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF 
;THE POSSIBILITY OF SUCH DAMAGE.
;------------------------------------------------------------